create or replace PROCEDURE        INSERT_IVR_ALERT_ACTION (
     I_ALERT_ID	IN NUMBER,
     I_MEMO_TEXT	IN VARCHAR2,
     I_INVALID_CONTACT_DETAILS_IND	IN VARCHAR2,
     I_CONTACT_HOME_PHONE_IND	IN VARCHAR2,
     I_CONTACT_WORK_PHONE_IND	IN VARCHAR2,
     I_CONTACT_MOBILE_PHONE_IND	IN VARCHAR2,
     I_USER_NAME	IN VARCHAR2,
     I_USER_ID	IN NUMBER,
     I_NEED_TO_COMMIT	IN VARCHAR2,
     I_FULFILL_SENT_CODE	IN VARCHAR2,
     O_ALERT_AVAILABLE	OUT VARCHAR2,
     O_SQLCODE	OUT NUMBER,
     O_SQLSTATE	OUT VARCHAR2 )

-----------------------------------------------------------------------------
-- File:  INSERT_IVR_ALERT_ACTION
--
--
-- Description:
--
--
--
-- Called By:
--     n/a
--
-- Calls:
--      GET_ALERT_ROW
--      SAVE_ALERT_FULFIL
--
-- Notes:
--		Save an IVR alert action record
--      Called by the webservice, so we don't validate the users access to the alert
--
-- Usage:
--		Called from the web Service
--
--
-- History:
--     2012/07/23   leushe	Ported from DB2
--     2013/03/12   leushe  Change support to alert/entity values of 100 bytes
--     2014/03/17   leushe  Added reference for FRAUD_SCORE_2-4 and remove FRAUD_REASON_CODES
--     2014/05/27   leushe  Changes for Configurable call results
--     2014/10/05   leushe  Changed the servicable check to be coalesce(l_result_type,'UNKNOWN') in ('POSITIVE','UNKNOWN') and NEXT_SERVICE_TIMESTAMP < l_now
--     2015/08/29    leushe      Added init_transaction_amt,    init_transaction_timestamp,	 o_init_transaction_mod_amt ,    o_lst_transaction_mod_amt support
--     2017/03/16    leushe   Captured NO_DATA_FOUND exception when result_id is null and looking up result_type 
--
------------------------------------------------------------------------
IS

	  	  l_exists number default 0;
        l_notexists number default 0;
		 l_alert_value VARCHAR2(100);
	  	   L_PARENT_ALERT_VALUE VARCHAR2(100) ;
	  	   L_PARENT_ALERT_TYPE VARCHAR2(1) ;
	  	   L_CONTACT_VALUE VARCHAR2(100) ;
	  	   L_CONTACT_TYPE VARCHAR2(1) ;
	  	   l_TENANT_multi_org_id integer;
	  	   l_alrt_ver_nbr integer ;
	  	   l_strategy_id NUMBER ;
		   l_queue_id NUMBER ;
		   l_multi_org_id integer;
           l_alert_type VARCHAR2(1);
		   l_result_id  integer;
		   l_result_type varchar2(10);
		   l_alert_status VARCHAR2(10) ;
		   l_gmt_offset NUMBER DEFAULT 0.0;--decimal(3,1) DEFAULT 0.0;
		   l_fraud_type_code  VARCHAR2(3);
	       l_fraud_score_1  integer;
	       l_fraud_score_2  integer;
	       l_fraud_score_3  integer;
	       l_fraud_score_4  integer;
	       l_init_fraud_score_1 integer;
	       l_init_fraud_score_2 integer;
	       l_init_fraud_score_3 integer;
	       l_init_fraud_score_4 integer;    
	       l_hwm_fraud_score_1  integer;
	       l_hwm_fraud_score_2  integer;
	       l_hwm_fraud_score_3  integer;
	       l_hwm_fraud_score_4  integer;    
	       l_alert_service_cnt  integer  ;
		   l_alert_origin VARCHAR2(20) ;
		   l_lst_transaction_amt  BINARY_DOUBLE;--decimal(19,5);
		   l_lst_transaction_timestamp  timestamp;
		   l_user_service_timestamp timestamp;
		   l_system_service_timestamp timestamp;
		   l_next_service_timestamp timestamp;
		   l_queue_entry_timestamp  timestamp;
		   l_final_disposition_timestamp  timestamp;
		   l_checkout_user  VARCHAR2(60) ;
		   l_checkout_timestamp timestamp;
		   l_create_user  VARCHAR2(60) ;
		   l_create_timestamp timestamp;
		   l_lstupdt_user VARCHAR2(60);
		   l_lstupdt_timestamp  timestamp;
		   l_alert_version  smallint ;
		   l_logical_del_ind  VARCHAR2(1);
		   l_action_type VARCHAR2(20) DEFAULT 'IVR ACTION';
		   l_base_rule_id NUMBER ;
		   l_rule_version_nbr integer ;
		   l_pause_timestamp timestamp;
		   l_pause_elapsed_time integer;
		   l_alert_action_id NUMBER ;
		 l_sys_block_code_1 VARCHAR2(8);
		 l_trans_block_code VARCHAR2(8);
		 l_resurface_timestamp timestamp;
		 l_sent_secure_message_ind VARCHAR2(1) DEFAULT '0';
		 l_failed_contact_ind VARCHAR2(1);
		 l_invalid_contact_ind VARCHAR2(1);
	     l_now TIMESTAMP;
     	 l_init_transaction_amt  BINARY_DOUBLE;
	 	 l_init_transaction_timestamp  timestamp;	     
	 	 l_init_transaction_mod_amt binary_double; 
	 	 l_lst_transaction_mod_amt binary_double;	
BEGIN
		l_now := SYS_EXTRACT_UTC(CURRENT_TIMESTAMP);

	  -- Get copy of required alert_action field from the alert table

	   GET_ALERT_ROW
         ( i_alert_id
          ,l_alrt_ver_nbr
          ,l_alert_value
          ,l_parent_alert_value
          ,l_parent_alert_type
          ,l_contact_value
          ,l_contact_type
          ,l_TENANT_multi_org_id 
          ,l_queue_id
          ,l_multi_org_id
          ,l_alert_type
          ,l_result_id
          ,l_alert_status
          ,l_gmt_offset
          ,l_fraud_type_code
          ,l_fraud_score_1
          ,l_fraud_score_2
          ,l_fraud_score_3
          ,l_fraud_score_4
          ,l_init_fraud_score_1
          ,l_init_fraud_score_2
          ,l_init_fraud_score_3
          ,l_init_fraud_score_4          
          ,l_hwm_fraud_score_1
          ,l_hwm_fraud_score_2
          ,l_hwm_fraud_score_3
          ,l_hwm_fraud_score_4
          ,l_alert_service_cnt
          ,l_alert_origin
          ,l_lst_transaction_amt
          ,l_lst_transaction_timestamp
          ,l_user_service_timestamp
          ,l_system_service_timestamp
          ,l_next_service_timestamp
          ,l_queue_entry_timestamp
          ,l_final_disposition_timestamp
          ,l_checkout_user
          ,l_checkout_timestamp
          ,l_create_user
          ,l_create_timestamp
          ,l_lstupdt_user
          ,l_lstupdt_timestamp
          ,l_alert_version
          ,l_base_rule_id
          ,l_rule_version_nbr
          ,l_pause_timestamp
          ,l_pause_elapsed_time
          ,l_init_transaction_amt
		  ,l_init_transaction_timestamp   
		  ,l_init_transaction_mod_amt  
		  ,l_lst_transaction_mod_amt 			  
          );

	  select strategy_id into l_strategy_id from fcm_queue where queue_id = l_queue_id;
      Select ALERT_ACTION_SEQ.nextval into L_ALERT_ACTION_ID from DUAL;
	  BEGIN
        select result_type into l_result_type from fcm_call_result where result_id=l_result_id;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN  l_result_type := 'UNKNOWN';
      END;
--select result_type into l_result_type from fcm_call_result where result_id=l_result_id;

	  -- Insert new alert action
	  INSERT INTO FCM_ALERT_ACTION(
					ALERT_ACTION_ID,
					ALERT_ID,
					ALERT_VERSION_NBR,
					ACTION_TIMESTAMP,
					PARENT_ALERT_VALUE,
					PARENT_ALERT_TYPE,
					ALERT_VALUE,
					CONTACT_VALUE,
					CONTACT_TYPE,
					TENANT_MULTI_ORG_ID,
					QUEUE_ID,
					MULTI_ORG_ID,
					ALERT_TYPE,
					RESULT_ID,
					ALERT_STATUS,
					GMT_OFFSET,
					FRAUD_TYPE_CODE,
		          FRAUD_SCORE_1,
		          FRAUD_SCORE_2,
		          FRAUD_SCORE_3,
		          FRAUD_SCORE_4,
		          INIT_FRAUD_SCORE_1,
		          INIT_FRAUD_SCORE_2,
		          INIT_FRAUD_SCORE_3,
		          INIT_FRAUD_SCORE_4 ,         
		          HWM_FRAUD_SCORE_1,
		          HWM_FRAUD_SCORE_2,
		          HWM_FRAUD_SCORE_3,
		          HWM_FRAUD_SCORE_4,
					ALERT_SERVICE_CNT,
					ALERT_ORIGIN,
					LST_TRANSACTION_AMT,
					LST_TRANSACTION_TIMESTAMP,
					USER_SERVICE_TIMESTAMP,
					SYSTEM_SERVICE_TIMESTAMP,
					NEXT_SERVICE_TIMESTAMP,
					QUEUE_ENTRY_TIMESTAMP,
					FINAL_DISPOSITION_TIMESTAMP,
					CHECKOUT_USER,
					CHECKOUT_TIMESTAMP,
					CREATE_USER,
					CREATE_TIMESTAMP,
					LSTUPDT_USER,
					LSTUPDT_TIMESTAMP,
					LSTUPDT_NBR,
					CONTACT_HOME_PHONE_IND,
					CONTACT_MOBILE_PHONE_IND,
					CONTACT_WORK_PHONE_IND,
					RESURFACE_TIMESTAMP,
					ACTION_TYPE,
					BASE_RULE_ID,
					RULE_VERSION_NBR,
					PAUSE_TIMESTAMP,
					PAUSE_ELAPSED_TIME,
					STRATEGY_ID,
          			INIT_TRANSACTION_AMT,
		  			INIT_TRANSACTION_TIMESTAMP, 
		  			INIT_TRANSACTION_MOD_AMT,  
		  			LST_TRANSACTION_MOD_AMT 		  			
					)
					VALUES(
					L_ALERT_ACTION_ID,
					i_alert_id,
					l_alrt_ver_nbr,
					l_now ,
					l_parent_alert_value,
					l_parent_alert_type,
					l_alert_value,
					l_contact_value,
					l_contact_type,	
				    l_TENANT_multi_org_id,
					l_queue_id,
					l_multi_org_id,
					l_alert_type,
					0 ,-- result_id
					l_alert_status,
					l_gmt_offset,
					l_fraud_type_code ,
		          l_fraud_score_1,
		          l_fraud_score_2,
		          l_fraud_score_3,
		          l_fraud_score_4,
		          l_init_fraud_score_1,
		          l_init_fraud_score_2,
		          l_init_fraud_score_3,
		          l_init_fraud_score_4 ,         
		          l_hwm_fraud_score_1,
		          l_hwm_fraud_score_2,
		          l_hwm_fraud_score_3,
		          l_hwm_fraud_score_4,
					l_alert_service_cnt ,
					l_alert_origin,
					l_lst_transaction_amt ,
					l_lst_transaction_timestamp ,
					l_user_service_timestamp,
					l_system_service_timestamp,
					l_next_service_timestamp,
					l_queue_entry_timestamp ,
					l_final_disposition_timestamp ,
					l_checkout_user ,
					l_checkout_timestamp,
					i_user_name ,
					l_now,
					i_USER_NAME,
					l_now,
					l_alert_version,
					i_contact_home_phone_ind,
					i_contact_mobile_phone_ind,
					i_contact_work_phone_ind,
					l_resurface_timestamp,
					l_action_type,
					l_base_rule_id,
					l_rule_version_nbr,
					l_pause_timestamp,
					l_pause_elapsed_time,
					l_strategy_id,
          			l_init_transaction_amt,
		  			l_init_transaction_timestamp,
		  			l_init_transaction_mod_amt,  
		  			l_lst_transaction_mod_amt 		  			
					);


	 --l_alert_action_id := identity_val_local();

	 SAVE_ALERT_FULFIL(
               l_alert_action_id,
               '0', -- i_SENT_LETTER_IND,
               '0', -- i_SENT_EMAIL_IND,
               '0',  -- i_SENT_SMS_IND,
               i_MEMO_TEXT
               ,i_USER_NAME
               ,l_SENT_SECURE_MESSAGE_IND
               ,i_INVALID_CONTACT_DETAILS_IND
               ,i_fulfill_sent_code
               ,NULL  --i_FULFIll_XML_MSG
               ,o_SQLCODE
               ,o_SQLSTATE
               );

	IF i_need_to_commit = 'Y' THEN
		COMMIT;
	END IF;

	 o_sqlcode := 0;
	 o_sqlstate := 'SUCES';



	select count(1) into l_notexists
				from fcm_alert_suspend fas
				where l_contact_value = fas.contact_value 
				    and l_contact_type = fas.contact_type
				    and l_TENANT_multi_org_id = fas.TENANT_multi_org_id
					and fas.suspend_user !=i_user_name
         			and rownum = 1;

 select count(1) into l_exists
						from fcm_strategy_user su, fcm_queue q, fcm_strategy s
								where q.queue_id = l_queue_id and
								      su.strategy_id = q.strategy_id and
								      s.strategy_id = q.strategy_id and
								      s.strategy_status = 'ACTIVE' and
									  q.queue_status in ('ACTIVE', 'QUIESCE') and
								      su.user_id = i_user_id and
                      rownum = 1;

 IF (l_notexists = 0 AND l_exists = 1			AND
				l_alert_status = 'ACTIVE'
			AND
            	coalesce(l_result_type,'UNKNOWN') in ('POSITIVE','UNKNOWN')
            AND 
            	l_next_service_timestamp < l_now) THEN

		 o_alert_available := '1';
	ELSE
		 o_alert_available := '0';
	END IF;
	update fcm_ivr_alert set update_nbr = update_nbr + 1 where alert_id = i_alert_id;
END INSERT_IVR_ALERT_ACTION;
/
show errors;


