CREATE OR REPLACE FUNCTION GET_VXX (i_tran_id numeric,i_proc_utc_datetime timestamp,i_table_name varchar2,i_uvr_id varchar2) 
         RETURN vxx_row_table pipelined IS
-----------------------------------------------------------------------------
-- File:  GET_VXX
--
--
-- Description:  For the given CMX_TRAN_ID,RQO_PROC_UTC_DATETIME, the function executes a query against the Table Name passed in
--               and returns the build_num and BLOB of the VXX data for the given UVR_IDs passed in.
--               The function then pivots the data into one row, of type vxx_row
--
--
-- Called By:    webapp Estimation process.
--
--
-- Calls:
--				decode_base64_VXX
--
-- Notes:
--
--
-- Usage:
--		with tran as (select distinct                                           /*---- USED DISTINCT HERE SO WE DONT GET REPEATING CMX_TRAN_IDS ----*/
--		  fcm_csca.CMX_TRAN_ID ,
--		  fcm_csca.rqo_proc_utc_datetime,
--		  RQO_SEG_ID_VERSION,
--		  CMX_MOD_AMT 
--		  from FCM_CSCA inner join fcm_csca_vxx                                   /*----- I DID THIS INNER JOIN SO THAT WE DONT LOOK FOR vxx RECORDS NOT HERE */
--		on fcm_csca.cmx_tran_id=fcm_csca_vxx.cmx_Tran_id
--		      and fcm_csca.rqo_proc_utc_datetime = fcm_csca_vxx.rqo_proc_utc_datetime
--		)
--		  select cast(TRAN.CMX_TRAN_ID as varchar2(20)) ,
--		  TRAN.RQO_SEG_ID_VERSION,
--		  TRAN.CMX_TRAN_ID,
--		  TRAN.CMX_MOD_AMT ,
--		  VXX.vxx00_build_num,
--		  VXX.vxx00,
--		  VXX.vxx01,
--		  VXX.vxx02,
--		  VXX.vxx03,
--		  VXX.vxx04,
--		  VXX.vxx05,
--		  VXX.vxx06,
--		  VXX.vxx07
--		  FROM
--		     TRAN
--		    INNER JOIN  ( TABLE(get_vxx(TRAN.CMX_TRAN_ID,TRAN.RQO_PROC_UTC_DATETIME,'FCM_CSCA_VXX','00,01')) )  VXX     /*--- GET_VXX FUNCTION HAS 3 ARGUMENTS. NOTE, IT ONLY SUPPORTS 8 UV SEGMENTS,*/
--		      ON TRAN.CMX_TRAN_ID=VXX.CMX_TRAN_ID;    
--
--
-- History:    2015/11/16:  Creation date
--			   2017/10/10:  Changed the Cursor query to pivot the I_uvr_id input variable list that is comma delimited
------------------------------------------------------------------------
TYPE         ref0 IS REF CURSOR;
cur0         ref0;
out_rec      vxx_row := vxx_row(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,
                  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,null);

uvr_id varchar2(2);
vxx_blob blob;
build_num numeric(10);


BEGIN

OPEN cur0 FOR
  'SELECT vxx_uvr_id,vxx_build_num,decode_base64_VXX (VARIABLE_DATA_00,VARIABLE_DATA_01,VARIABLE_DATA_02,VARIABLE_DATA_03,VARIABLE_DATA_04,VARIABLE_DATA_05,VARIABLE_DATA_06,VARIABLE_DATA_07,VARIABLE_DATA_08,VARIABLE_DATA_09,VARIABLE_DATA_10,VARIABLE_DATA_11)'
  ||' from '||i_table_name||' where cmx_tran_id = :1 and rqo_proc_utc_datetime = :2 and vxx_uvr_id in (select regexp_substr('''||i_uvr_id||''',''[^,]+'', 1, level) from dual
    connect by regexp_substr('''||i_uvr_id||''', ''[^,]+'', 1, level) is not null ) order by vxx_uvr_id '
USING i_tran_id,i_proc_utc_datetime;
  LOOP
    FETCH cur0 INTO uvr_id, build_num,   vxx_blob;
    EXIT WHEN cur0%NOTFOUND;
    case 
    WHEN (uvr_id='00') then         out_rec.vxx00:=vxx_blob;        out_rec.vxx00_build_num:=build_num;  
    WHEN (uvr_id='01') then         out_rec.vxx01:=vxx_blob;        out_rec.vxx01_build_num:=build_num;
    WHEN (uvr_id='02') then         out_rec.vxx02:=vxx_blob;        out_rec.vxx02_build_num:=build_num;
    WHEN (uvr_id='03') then         out_rec.vxx03:=vxx_blob;        out_rec.vxx03_build_num:=build_num;
    WHEN (uvr_id='04') then         out_rec.vxx04:=vxx_blob;        out_rec.vxx04_build_num:=build_num;
    WHEN (uvr_id='05') then         out_rec.vxx05:=vxx_blob;        out_rec.vxx05_build_num:=build_num;
    WHEN (uvr_id='06') then         out_rec.vxx06:=vxx_blob;        out_rec.vxx06_build_num:=build_num;
    WHEN (uvr_id='07') then         out_rec.vxx07:=vxx_blob;        out_rec.vxx07_build_num:=build_num;
    WHEN (uvr_id='08') then         out_rec.vxx08:=vxx_blob;        out_rec.vxx08_build_num:=build_num;  
    WHEN (uvr_id='09') then         out_rec.vxx09:=vxx_blob;        out_rec.vxx09_build_num:=build_num;
    WHEN (uvr_id='10') then         out_rec.vxx10:=vxx_blob;        out_rec.vxx10_build_num:=build_num;
    WHEN (uvr_id='11') then         out_rec.vxx11:=vxx_blob;        out_rec.vxx11_build_num:=build_num;
    WHEN (uvr_id='12') then         out_rec.vxx12:=vxx_blob;        out_rec.vxx12_build_num:=build_num;
    WHEN (uvr_id='13') then         out_rec.vxx13:=vxx_blob;        out_rec.vxx13_build_num:=build_num;
    WHEN (uvr_id='14') then         out_rec.vxx14:=vxx_blob;        out_rec.vxx14_build_num:=build_num;
    WHEN (uvr_id='15') then         out_rec.vxx15:=vxx_blob;        out_rec.vxx15_build_num:=build_num;
    WHEN (uvr_id='16') then         out_rec.vxx16:=vxx_blob;        out_rec.vxx16_build_num:=build_num;  
    WHEN (uvr_id='17') then         out_rec.vxx17:=vxx_blob;        out_rec.vxx17_build_num:=build_num;
    WHEN (uvr_id='18') then         out_rec.vxx18:=vxx_blob;        out_rec.vxx18_build_num:=build_num;
    WHEN (uvr_id='19') then         out_rec.vxx19:=vxx_blob;        out_rec.vxx19_build_num:=build_num;
    WHEN (uvr_id='20') then         out_rec.vxx20:=vxx_blob;        out_rec.vxx20_build_num:=build_num;
    WHEN (uvr_id='21') then         out_rec.vxx21:=vxx_blob;        out_rec.vxx21_build_num:=build_num;
    WHEN (uvr_id='22') then         out_rec.vxx22:=vxx_blob;        out_rec.vxx22_build_num:=build_num;
    WHEN (uvr_id='23') then         out_rec.vxx23:=vxx_blob;        out_rec.vxx23_build_num:=build_num;
    WHEN (uvr_id='24') then         out_rec.vxx24:=vxx_blob;        out_rec.vxx24_build_num:=build_num;  
    WHEN (uvr_id='25') then         out_rec.vxx25:=vxx_blob;        out_rec.vxx25_build_num:=build_num;
    WHEN (uvr_id='26') then         out_rec.vxx26:=vxx_blob;        out_rec.vxx26_build_num:=build_num;
    WHEN (uvr_id='27') then         out_rec.vxx27:=vxx_blob;        out_rec.vxx27_build_num:=build_num;
    WHEN (uvr_id='28') then         out_rec.vxx28:=vxx_blob;        out_rec.vxx28_build_num:=build_num;
    WHEN (uvr_id='29') then         out_rec.vxx29:=vxx_blob;        out_rec.vxx29_build_num:=build_num;
    WHEN (uvr_id='30') then         out_rec.vxx30:=vxx_blob;        out_rec.vxx30_build_num:=build_num;
    WHEN (uvr_id='31') then         out_rec.vxx31:=vxx_blob;        out_rec.vxx31_build_num:=build_num;
    WHEN (uvr_id='32') then         out_rec.vxx32:=vxx_blob;        out_rec.vxx32_build_num:=build_num;
    WHEN (uvr_id='33') then         out_rec.vxx33:=vxx_blob;        out_rec.vxx33_build_num:=build_num;        
    end case;
        
    
    EXIT WHEN cur0%NOTFOUND;
       
    
  END LOOP;
  
  CLOSE cur0;
  out_rec.cmx_tran_id:=i_tran_id;
  out_rec.stmt_text:='SELECT vxx_uvr_id,decode_base64_VXX (VARIABLE_DATA_00,VARIABLE_DATA_01,VARIABLE_DATA_02,VARIABLE_DATA_03,VARIABLE_DATA_04,VARIABLE_DATA_05,VARIABLE_DATA_06,VARIABLE_DATA_07,VARIABLE_DATA_08,VARIABLE_DATA_09,VARIABLE_DATA_10,VARIABLE_DATA_11)'
  ||' from '||i_table_name||' where cmx_tran_id = :1 and rqo_proc_utc_datetime = :2 and vxx_uvr_id in ('||i_uvr_id||') order by vxx_uvr_id';
  out_rec.rqo_proc_utc_datetime:=i_proc_utc_datetime;
    PIPE ROW(out_rec);
RETURN;
END get_vxx;
/