%macro cfpr_outsort_charts(driver=eval_current, base=CFPR, metrics=CDR VDR, interior_cellwidth=1.5in, title2= , report_levels=);

%delit(ds=merged_&base);
%numobs(eval_current);

%let prefix=%upcase(%substr(&base,1,1));
%let base=%upcase(&base);

%if &prefix=O %then %let keeplist=&base &metrics outsort_1000000 CDR VDR;
%else %let keeplist=&base &metrics CDR VDR;

*-----------------------------------------------------------------------*
|  Note:  This macro has not been 'generalized' to run on metrics other |
|         than CDR/VDR.  It can run on both, or either one but not on   |
|         other metrics.  It could be generalized by modifying the      |
|         keeplist and labels to loop through metric names              |
*-----------------------------------------------------------------------*;

*-----------------------------------------------------------------------*
|            Merge the Eval Data Sets for Tables                        |
*-----------------------------------------------------------------------*;

data _null_;
 set eval_current end=last;

newname="&prefix._"||substr(memname,1,28);
date1_char=put(date1,date9.);
date2_char=put(date2,date9.);
suffix=cats(date1_char,"_",date2_char);

 if  _n_=1 then do;
	 call execute("Data merged_&base;");
	 call execute("length &base Report_Level 8 label_VDR label_CDR $30;");
	 call execute("Merge ");
 end;
    if report_type='EVAL' then call execute("&qtrlib.."||strip(newname)||"(keep=&keeplist rename=(VDR=VDR_"||suffix||" CDR=CDR_"||suffix||"))");
	else if report_type='DEV' then call execute("&qtrlib.."||strip(newname)||"(keep=&keeplist rename=(VDR=VDR_DEV CDR=CDR_DEV))");
 
 if last then do;
    call execute(";");
	%if &prefix=O %then %do; call execute("by outsort_1000000;");
							 call execute("outsort=outsort_1000000/1000000;");
	                         call execute("format outsort percent8.2;");
							 call execute("outsort_char=put(outsort,percent8.2);");
							 call execute("label_VDR='VDR at &base level '||strip(outsort_char);");
	                         call execute("label_CDR='CDR at &base level '||strip(outsort_char);");
							 call execute("drop  outsort_1000000;"); 
							 
						%end;
	%else %do; call execute ("by &base;");;
			   call execute("label_VDR='VDR at &base level '||strip(&base)||':1';");
	           call execute("label_CDR='CDR at &base level '||strip(&base)||':1';");
	%end;
	call execute("Report_Level=0;");
    call execute("do j=&report_levels;");
	call execute("if abs(&base-j)<0.00001 then Report_Level=1;");
	call execute("end;");
	call execute("drop j;");
	call execute("run;");
 end;

run;

*-----------------------------------------------------------------------*
|    Add labels to merged dataset                                       |
*-----------------------------------------------------------------------*;

data qqq_&base;
   set sashelp.vcolumn(keep=name libname memname where=(libname='WORK' and memname="MERGED_&base"));
	 node1=scan(name,1,"_");
	 node2=scan(name,2,"_");
	 node3=scan(name,3,"_");
	 date1=input(node2,?? date9.);
	 date2=input(node3,?? date9.);
	 length label $32;
	      if date1 ne . then label="Prod "||put(date1,custdate.)||" to "||put(date2,custdate.);
	 else if node1='label' then label="Metric";
	 else if node2='DEV' then do;
		label="Dev &devb to &deve"; 
	 end;
	 drop node1-node3 date1-date2 libname memname;
 run;

data _null_;
 set qqq_&base(where=(missing(label)=0)) end=last;
	if _n_=1 then do;
	 call execute("proc datasets library=WORK nolist nodetails;");
	 call execute("modify MERGED_&base;");
	 call execute("label ");
	end;

	call execute(strip(name)||"="||quote(strip(label)));

	if last then do;
	call execute("; run; quit;");
	end;
run;

*-----------------------------------------------------------------------*
|    For the Base - create 1 table for each Metric                      |
*-----------------------------------------------------------------------*;

%let loopnum=1;
%let loopvar=%scan(&metrics,1);
%do %while(&loopvar ne );

proc sql noprint;
 select name
 into :vname1-:vname%left(%eval(&numobs+1))
 from dictionary.columns
 where libname='WORK' and memname="MERGED_&base" and name contains "&loopvar";

data merged_&base;
 set merged_&base;
 holder=1;
run;

ods proclabel "&loopvar by &base Table";

proc report data=merged_&base split=' to ' style(header)=[verticalalign=bottom] contents="Key Metric Report";
where report_level=1;
columns %do j=1 %to %left(%eval(&numobs+1)); &&vname&j %end; holder;
define label_&loopvar/display  style(column)=[cellwidth=2.0in just=left];
%do j=2 %to %left(%eval(&numobs+1));
define &&vname&j/display format=10.2 style(column)=[cellwidth=&interior_cellwidth];
%end;
define holder/order noprint;
break before holder/page contents='';
title "&loopvar by &base Time Series Tables";
%if %bquote(&title2) ne %str() %then title2 "&title2";;
run;


%let loopnum=%eval(&loopnum+1);
%let loopvar=%scan(&metrics,&loopnum);
%end;


%delit(ds=merged_&base);
%delit(ds=qqq_&base);

%mend cfpr_outsort_charts;
