%macro dateprep_quarterly;

%global
    development_begin_date
    development_end_date
    TSTART
    TEND
    eval_win_begin_date
    eval_win_end_date
    extract_wherecond
    extract_wherecond_rh;

%local report_date_raw;
%let report_date_raw = &report_date;

%if &report_date_raw eq  %then %do;
    %let report_date = %sysfunc(today());
%end;
%else %do;
    %let report_date = "&report_date_raw"d;
%end;


%if %in1(%lowcase(&type), dev_quarterly) %then %do;
    %let development_begin_date = %sysget(dev_beginning);
    %let development_end_date = %sysget(dev_end);
    data qtrlib.development_dates;
        development_begin_date = "&development_begin_date"d;
        development_end_date = "&development_end_date"d;
        format development_begin_date development_end_date date9.;
    run;
%end;

%if %sysfunc(exist(qtrlib.development_dates)) %then %do;
    data _null_;
        set qtrlib.development_dates;
        call symputx('development_begin_date', put(development_begin_date, best.), 'G');
        call symputx('development_end_date', put(development_end_date, best.), 'G');
    run;
%end;
%else %do;
    %let development_begin_date = %str(.);
    %let development_end_date = %str(.);
%end;

data _null_;

    rundate=today();
    report_date=&report_date;

    if "%lowcase(&type)" ne "dev_quarterly" then do;
        if "&report_date_raw" eq "." then do;
            eval_win_end_date = intnx('qtr', report_date, -2, 'end');
        end;
        else do;
            drop tmp_end_date;
            tmp_end_date = intnx('month', report_date, 0, 'end');
            eval_win_end_date = ifn(tmp_end_date - report_date <= 1, tmp_end_date,
                                    intnx('month', report_date, -1, 'end'));
        end;
        eval_win_begin_date = intnx('month', eval_win_end_date, -&eval_win_num_months + 1, 'begin');
    end;
    else do;
        eval_win_end_date = &development_end_date;
        eval_win_begin_date = &development_begin_date;
    end;

    call symputx('rundate', rundate, 'G');

    call symputx('eval_win_begin_date', eval_win_begin_date, 'G');
    call symputx('eval_win_end_date', eval_win_end_date, 'G');

    call symputx('extract_wherecond',
        catx(" ", "rqo_tran_date >=", put(eval_win_begin_date, best.),
                  "and", "rqo_tran_date <=",  put(eval_win_end_date, best.)),
        'G');


    /* Indexes are created on rhx_date_key (date part of rqo_proc_utc_datetime)
     * Subsetting based on rhx_date_key to speed up the process
     * We also allow 15 days buffer time before and after evaluation window
     * in order to better estimate date of frist fraud and blocking from RHX tags
     * in exclusion_logic, we first subsetting by rqo_tran_date
     */
    extract_buffer_days = 15;
    %if %lowcase(&rh_db_source) ne oracle %then %do;
        call symputx('extract_wherecond_rh',
            catx(" ", "rqo_tran_date >=", put(eval_win_begin_date - extract_buffer_days, best.),
                      "and", "rqo_tran_date <=",  put(eval_win_end_date + extract_buffer_days, best.)),
            'G');
    %end;

    %else %do;
        call symputx('extract_wherecond_rh',
            catx(" ", "rhx_date_key >=", put((eval_win_begin_date - extract_buffer_days)*86400, best.),
                      "and", "rhx_date_key <=",  put((eval_win_end_date + extract_buffer_days)*86400, best.)),
            'G');
    %end;
run;

%let TSTART=%sysfunc(putn(&eval_win_begin_date, DATE9.));
%let TEND=%sysfunc(putn(&eval_win_end_date, DATE9.));

%put report_date: %sysfunc(putn(&report_date, worddate.));
%put evaluation window begin: %sysfunc(putn(&eval_win_begin_date, worddate.));
%put evaluation window end: %sysfunc(putn(&eval_win_end_date, worddate.));

%mend dateprep_quarterly;
