%macro main_report_outputds;

*---------------------------------------------------------------------------------------*
|   Main Report - no by variable - name of PDF is main report                           |
*---------------------------------------------------------------------------------------*;; 

      %if %in1(&type,DEV_DAILY DEV_MONTHLY) %then %let type_long=Development;
%else %if &TYPE eq DAILY %then %let type_long=Daily;
%else %if &TYPE eq MONTHLY %then %let type_long=Monthly;

proc sql noprint;
 select label
 into :label1-:label%left(&time_periods)
 from date_range
 order by seqno;

proc sort data=&config_ds;
 by seq_original;
run;

data _null_;
 set &config_ds end=last;

 by report_variable drill_down_variable NOTSORTED;

 *------------------- TURN OFF STABILITY INDEX IN THIS SECTION -------------------*;
  stability_index='N';

/** the following needed for card-level variables **/

aggregation_type=upcase(aggregation_type);
length drill_number time_periods 8;
time_periods=strip(&time_periods);
drill_number=scan(drill_vname,3,"_");
length tablecount 8;
tablecount=&tablecount;

TYPE=strip("&TYPE");  **create type as a ds variable to resolve timing issues;
frh_table=strip("&frh_table");

proc_report_statistic=strip("%upcase(&proc_report_statistic)"); **proc report statistic for aggregated variables - always sum for now***;

     if type in('DEV_DAILY','DEV_MONTHLY')     then outputds_prefix='Dev    ';
else if type='DAILY'   then outputds_prefix='Daily  ';
else if type='MONTHLY' then outputds_prefix='Monthly';


if upcase(type) in('DEV_DAILY','DEV_MONTHLY') then Output_libref='Repout';
                                              else Output_libref='Work';


    X='if _break_ ne " " then call define("'||strip(Report_Variable)||'","style","style=[pretext='||"'Total:']"||'");';
         *** coding the above as a character - the combinations of quotes get tricky within call execute;
    X2='if _break_ ne " " then call define("group","style","style=[pretext='||"'Total:']"||'");';
         *** coding the above as a character - the combinations of quotes get tricky within call execute - use for C-level;

    title=strip("&title"); ***&title will not resolve in single quotes within call execute and title statement;
        title_rep_var_lab=translate(Report_Variable_Label," ","|");  *** need to remove split character from title ***;

/*** define stability index coding ***/
if upcase(stability_index) in('P','D') then do;
      stab_index_col_var="stability_index"; 
          cindex1=&cindex1;
          if upcase(stability_index)='P' then do; stab_period="Previous   "; cindex2=&cindex2_P; end;
          else if upcase(stability_index)='D' then do; stab_period="Development"; cindex2=&cindex2_D; end;
end;  
else stab_index_col_var=" ";

/*** define prefix for output data set - either dev, daily, monthly ***/
   call execute("ods escapechar='~';");
 
*--------------------------------------------------------------------------------------------------------------------*
|  Main Report - PROC REPORT Generation for type 'T' is below                                                        |
|  The code differences between this and types 'A' and 'R' are as follows:                                           |
|      1.) 'T' uses raw input data -- 'A' and 'R' use aggregated data from PROC SQL                                  |
|      2.) 'T' uses multilabel date format as an across variable -- 'A' and 'R' have date data summarized and merged |
|      3.) 'T' produces only counts -- 'A' and 'R' allow for sums of variables                                       |
*--------------------------------------------------------------------------------------------------------------------*;

**** programming note - the code below for each type can be simplified - but it is more readable for now - efficiency is not lost ****;

**** all runs produce output data sets - but only Dev and Production Runs where no dev data exists produce a pdf;
**** final reports are NOT run for production data without dev data - final reports use the output ds functionality instead;

   if (type in('DEV_DAILY','DEV_MONTHLY') OR Tablecount=0)  AND _N_=1 THEN DO; **only create output here for dev ***;         
        call execute("ods pdf body='&report_path"||strip(type)||"_"||strip(frh_table)||"_&report_date9."||".pdf' contents=NO  pdftoc=1 style=&mainstyle;");
   END;


 IF aggregation_type='N' then do;

 if first.report_variable then do;
     drill_level=0; 
      
        
           call execute("ods proclabel '"||strip(report_variable)||": Distribution Report';");
          call execute("proc report data=&output_recode_ds split='|' missing  NOcompletecols completerows nowindows ");
		  call execute(" contents='Distribution Report' ");
if type in('DEV_DAILY','DEV_MONTHLY') 
then call execute("out=repout."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";"); *** need to update 'repout.' to 'work.'  for daily monthly****;
              else call execute("out=work."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";"); *** need to update 'repout.' to 'work.'  for daily monthly****;
            call execute('title1 "~S={foreground=green}'||TITLE||" "||strip(type)||" "||strip(frh_table)||" Report - &report_date9. - "||strip(title_rep_var_lab)||" "||strip(Report_Variable)||'";'); 
          call execute("title2;");
        end;

        **** the code below creates a title link for each drill down variable ****;
                if type in('DEV_DAILY','DEV_MONTHLY') OR Tablecount=0  THEN DO; ***only create linked titles for dev - for this run ***;
        if drill_down_variable ne " " then do;
                drill_level+1;
                title_number=drill_level+2;
                titlelink='LINK="'||strip(Type)||"_"||strip(frh_table)||"_&report_date9._"||strip(Report_Variable)||"_Drill_BY_"||strip(Drill_Down_variable)||'.pdf" justify=L "Drill by: '||strip(Drill_Down_Label)||'";';
                *** coding the above as a character - the combinations of quotes get tricky within call execute;
                call execute("title"||strip(title_number)||" "||titlelink);
        end;
                else title_number=1;
                END;

        if last.report_variable then do;

        call execute("freq wts_var;"); ****verify correct result is obtained ****************;
        call execute("columns "||strip(Report_Variable)||" rqo_tran_date,(N PCTN) "||strip(stab_index_col_var)||";");

        call execute("define rqo_tran_date/across format=Date_range. mlf 'Date Ranges' ;");
   call execute("define "||strip(Report_Variable)||" /group  format="||strip(Report_Variable_Format)||" ORDER=INTERNAL preloadfmt  '"||strip(Report_Variable_Label)||"'  style(column)=[ just=left];");

        call execute("define N/format=comma15. 'Count' style(column)=[cellwidth=&cellwidth];");
        call execute("define PCTN/format=percent10.2 'Percent' style(column)=[cellwidth=&cellwidth];");

        call execute("compute "||strip(Report_Variable)||";");
        call execute(x); *** see character variable definition above;
        call execute(" endcomp;");

        call execute(" rbreak after/summarize style=[background=grey foreground=yellow] ;"); 
            if upcase(stability_index) in('P','D') then do;
                        call execute("define stability_index/computed 'Stability|Index|(Current vs. "||strip(stab_period)||" Period)' format=6.4 style(column)=[cellwidth=&cellwidth backgroundcolor=traffic.];");
                        call execute("compute stability_index;");
						    call execute("if (_c"||strip(cindex1)||"_ gt 0) AND  (_c"||strip(cindex2)||"_ gt 0) then");
                            call execute("stability_index=(_c"||strip(cindex1)||"_-_c"||strip(cindex2)||"_)*log(_c"||strip(cindex1)||"_/_c"||strip(cindex2)||"_);");
                            call execute("sumindices+stability_index;");
                                call execute("if _break_='_RBREAK_' then stability_index=sumindices;");
                                call execute("if _break_='_RBREAK_' then call define('Stability_index','style','Style=[backgroundcolor=traffic. color=black]');");
                call execute("endcomp;");

        end;
        call execute(" run;");
        end;
END;

*--------------------------------------------------------------------------------------------------------------------*
|  Main Report PROC REPORT Generation for types 'A' and 'R' are Below                                                |
|  The code differences between this and type 'T' are as follows:                                                    |
|      1.) 'T' uses raw input data -- 'A' and 'R' use aggregated data from PROC SQL                                  |
|      2.) 'T' uses multilabel date format as an across variable -- 'A' and 'R' have date data summarized and merged |
|      3.) 'T' produces only counts -- 'A' and 'R' allow for sums of variables                                       |
*--------------------------------------------------------------------------------------------------------------------*;

IF aggregation_type in('G','R') then do;

     length rv_short_name $27 ;
     rv_short_name=report_variable; DROP rv_short_name;

         array one{*} $ 32 vname1-vname%sysfunc(strip(&time_periods));

         do j=1 to dim(one);
           one{j}=cats(rv_short_name,"_TP_",j);
         end;


        if first.report_variable then do;
          drill_level=0; 
                

           call execute("ods proclabel '"||strip(report_variable)||": Distribution Report';");
          call execute("proc report data=work.Report_in_clev_var"||strip(varcount)||" split='|' missing  NOcompletecols completerows nowindows ");
		   call execute(" contents='Distribution Report' ");
if type in('DEV_DAILY','DEV_MONTHLY') 
then call execute("out=repout."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";"); *** need to update 'repout.' to 'work.'  for daily monthly****;
              else call execute("out=work."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";"); *** need to update 'repout.' to 'work.'  for daily monthly****;
call execute('title1 "~S={foreground=green}'||TITLE||" &type_long Report Report - &report_date9. - "||strip(title_rep_var_lab)||" "||strip(Report_Variable)||'";'); 
          call execute("title2;");
          /***call execute("freq wts_var;");***** weighting should occur in SQL !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*****************/
        end;
        

        **** the code below creates a title link for each drill down variable ****;
                if type in('DEV_DAILY','DEV_MONTHLY') OR Tablecount=0  THEN DO;
                if drill_down_variable ne " " then do;
                        drill_level+1;
                        title_number=drill_level+2;
                        titlelink='LINK="'||strip(Type)||"_"||strip(frh_table)||"_&report_date9._"||strip(Report_Variable)||"_Drill_BY_"||strip(Drill_Down_variable)||'.pdf" justify=L "Drill by: '||strip(Drill_Down_Label)||'";';
                        *** coding the above as a character - the combinations of quotes get tricky within call execute;
                        call execute("title"||strip(title_number)||" "||titlelink);
                end;
                    else title_number=1;
                END;

        if last.report_variable then do;

        ********************put generated c-level code here ************************;

                *** for card_number aggregation - the right column is NOT the aggregation variable - it is just a count ***;
             if aggregation_type='R' then  call execute("columns "||strip(report_variable)||"=group ('Date Ranges'");
    else if aggregation_type='G' then  call execute("columns "||strip(aggregation_variable)||"=group ('Date Ranges'");

        do j=1 to dim(one);
         call execute(one{j});
        end;

                call execute("),("||strip(Proc_Report_Statistic)||" PCT"||strip(Proc_Report_Statistic) ||") "||strip(stab_index_col_var)||";");

            if strip(PROC_REPORT_STATISTIC)="N"  then do; 
           call execute("define N/format="||strip(aggregation_result_format)||" 'Count' style(column)=[cellwidth=&cellwidth];");
               call execute("define PCTN/format=percent10.2 'Percent' style(column)=[cellwidth=&cellwidth];");
                end;

        if strip(PROC_REPORT_STATISTIC)="SUM" then do;
           call execute("define SUM/format="||strip(aggregation_result_format)||" '"||strip(aggregation_result_label)||"' style(column)=[cellwidth=&cellwidth];");
                   call execute("define PCTSUM/format=percent10.2 'Percent' style(column)=[cellwidth=&cellwidth];");
                 end;


        call execute("define group /group  format="||strip(report_variable_format)||" ORDER=INTERNAL preloadfmt '"||strip(report_variable_label)||"' style(column)=[  just=left];");
    call execute("compute group;");
    call execute(x2); *** see character variable definition above;
    call execute("endcomp;");
    call execute("rbreak after/summarize style=[background=grey foreground=yellow] ;");
        if upcase(stability_index) in('P','D') then do;
                        call execute("define stability_index/computed 'Stability|Index|(Current vs. "||strip(stab_period)||" Period)' format=6.4 style(column)=[cellwidth=&cellwidth backgroundcolor=traffic.];");
                        call execute("compute stability_index;");
						    call execute("if (_c"||strip(cindex1)||"_ gt 0) AND  (_c"||strip(cindex2)||"_ gt 0) then");
                            call execute("stability_index=(_c"||strip(cindex1)||"_-_c"||strip(cindex2)||"_)*log(_c"||strip(cindex1)||"_/_c"||strip(cindex2)||"_);");
                            call execute("sumindices+stability_index;");
                                call execute("if _break_='_RBREAK_' then stability_index=sumindices;");
                                call execute("if _break_='_RBREAK_' then call define('Stability_index','style','Style=[backgroundcolor=traffic. color=black]');");
                call execute("endcomp;");

    end;
    call execute("run;");
        ********************put generated c-level code here ************************;

        end;
END;


*--------------------------------------------------------------------------------------------------------------------*
|  Add Percentile Reports Here                                                                                       |
*--------------------------------------------------------------------------------------------------------------------*;

 if last.report_variable and substr(report_variable_format,1,1) NE '$' and upcase(percentile_report)='Y'  then do;

                 IF aggregation_type='N' then do;
                                call execute("ods proclabel '"||strip(report_variable)||": Percentile Report';");
                                call execute("proc tabulate data=&output_recode_ds");
								call execute(" contents='Percentile Report' ");
								call execute("out=Work.Tab_Var"||strip(varcount));
						        call execute(";");
                                call execute("freq wts_var;"); ****verify correct result is obtained ****************;
                                call execute("var "||strip(report_variable)||";");
                                call execute("class rqo_tran_date/mlf;");
                                call execute("format rqo_tran_date Date_range.;");
                                call execute("Tables (N*f=comma10. Nmiss Min P1 P5 P10 P25 Median P75 P95 P99 Max)*"||strip(report_variable)||"=' ',");
                                call execute("        (rqo_tran_date=' ')/row=float box='"||strip(Report_Variable_Label)||"';");
                                call execute("run;");
				  ****** the following processes the output dataset *****;
			 
							    call execute("data Work.Tab_Var"||strip(varcount)||";");

							    call execute("if _n_=1 then do;");  *** add the seqno to be used for transposing variable names ***;
							    call execute("if 0 then set date_range(keep=seqno label);");
							    call execute("declare hash tp(dataset: 'date_range(keep=seqno label rename=(label=rqo_tran_date))');");
							    call execute("tp.defineKey('rqo_tran_date');");
							    call execute("tp.defineData('seqno');");
							    call execute("tp.defineDone();");
							    call execute("end;");
							
						     	call execute("set Work.Tab_Var"||strip(varcount)||";");
		                        call execute("rc=tp.find(); drop rc label;");
							    call execute("run;");

								call execute("proc transpose data=Work.Tab_Var"||strip(varcount));
			                    call execute("out=Work.Tab_Var"||strip(varcount)||"_trans");
		                        call execute("(drop=_label_ where=(NOT(_name_ in('_PAGE_','_TABLE_')))) prefix=TP_;");
								call execute("id seqno;");
								call execute("idlabel rqo_tran_date;");
								call execute("run;");

								call execute("proc sql  noprint;");
								call execute(" select substr(_name_,1,index(_name_,cats('_',scan(_name_,-1,'_')))-1)");
								call execute(" into :name");
								call execute(" from Work.Tab_Var"||strip(varcount)||"_trans(obs=1);");
								call execute("quit;");
							
								call execute("data "||strip(Output_libref)||"."||strip(outputds_prefix)||"_tab_var_"||strip(varcount)||";");
								call execute('length %nrstr(&name) $32;');
								call execute(" set Work.Tab_Var"||strip(varcount)||"_trans;");
								call execute(' %nrstr(&name)=scan(_name_,-1,"_");');
								call execute(" drop _name_;");
								do j=1 to time_periods;
							 			IF type in('DEV_DAILY','DEV_MONTHLY') then call execute("_D"||strip(j)||"_=TP_"||strip(j)||";");
							 			else IF type in('DAILY','MONTHLY') then call execute("_C"||strip(j)||"_=TP_"||strip(j)||";");
                        		end;
								call execute("  run;");

			                    call execute("proc datasets library="||strip(Output_libref)||" nolist nodetails;");
								call execute("modify "||strip(outputds_prefix)||"_tab_var_"||strip(varcount)||";");
								call execute("label ");
								%do j=1 %to &time_periods;
								  if type in('DAILY','MONTHLY') then call execute("_C"||strip(&j)||"_='&&label&j' ");
		                          if type in('DEV_DAILY','DEV_MONTHLY') then call execute("_D"||strip(&j)||"_='&&label&j' ");
								%end;
								call execute(";");
								call execute("run; quit;");
                 END;

                        ****** DO NOT DO PERCENTILE REPORTS FOR TYPE='G'*******************************;
                 **IF aggregation_type in('G','R') then do;
                     IF aggregation_type in('R') then do;
                         **if aggregation_type='G' then ds=cats("Report_in_clev_var",strip(varcount));
                         if aggregation_type='R' then ds=cats("Percentile_in_clev_var",strip(varcount));
                         call execute("ods proclabel '"||strip(report_variable)||": Percentile Report';");
                         call execute("proc tabulate data=work."||strip(ds)||" contents='Percentile Report' ;");
                         if aggregation_type='R' then call execute("freq wts_var;"); /**** TYPE 'R' is not weighted for this ds because interim data is used *****/
                         call execute("var ");
                         do j=1 to dim(one); ** generates the time period vars **;
                             call execute(one{j});
                         end;
                         call execute(";");
                         call execute("tables");       
                         call execute("(N*f=comma10. Nmiss Min P1 P5 P10 P25 Median P75 P95 P99 Max),");
                         call execute("(");
                         do j=1 to dim(one); ** generates the time period vars **;
                             call execute(one{j});
                         end;
                         call execute(")/Box='"||strip(Report_Variable_Label)||"';");
                         call execute("run;");

						 /***
	 					 ** if type is dev - create the tabulate output to 'put on a shelf'- otherwise output dataset   goes to work. ***;
					     **** this is the output tabulate data set for type 'R' (not run for type 'G') - we don't need it for production reports without dev data;
						 **** final reports are NOT run for production data without dev data - final reports use the output ds functionality instead;
						 **** otherwise we need to produce the outputds for all Production Runs with Dev Data and all Dev runs;***/

						 IF NOT( type in('DAILY','MONTHLY') and Tablecount=0) then do;

			                 call execute("proc means data=work."||strip(ds)||" stackodsoutput  N Nmiss Min P1 P5 P10 P25 Median P75 P95 P99 Max;");
							 call execute("freq wts_var;"); /**** TYPE 'R' is not weighted for this ds because interim data is used *****/
							 call execute("var ");
		                      do j=1 to dim(one); ** generates the time period vars **;
		                         call execute(one{j});
		                      end;
		                     call execute(";");
							 call execute("ods output summary=work.Tab_Var"||strip(varcount)||";");
							 call execute("run;");

							call execute("proc transpose data=work.Tab_Var"||strip(varcount));
		                    call execute("out="||strip(Output_libref)||"."||strip(outputds_prefix)||"_tab_var_"||strip(varcount));
		                    call execute("(drop=_label_ rename=(_name_="||strip(report_variable)||"));");
							call execute("id variable;");
							call execute("idlabel label;");
							call execute("run;");
						END;
	   
                     END;
        END;

if type in('DEV_DAILY','DEV_MONTHLY') and  last.report_variable then do;
        *** the following is in the Dev loop and only executes for Dev;
        call execute("Data repout."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";");
        call execute("Set repout."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";");
        call execute("Drop _BREAK_;");
        call execute("If _BREAK_='_RBREAK_' then delete;");
        call execute("rename _C2_=_D1_");
        call execute("       _C3_=_D2_;");
        IF aggregation_type in('G','R') then do; 
                call execute("label _C2_='"||strip(aggregation_result_label)||"';"); 
                call execute("rename group="||strip(report_variable)||";");
                call execute("format group " || strip(report_variable_format) || ";");
    	end;
        else do;
            call execute("format " || strip(report_variable) || " " || strip(report_variable_format) || ";");
            call execute("label _C2_='Count';");
		end;
        call execute("label  _C3_='Percent';");
        call execute("run;");   

       **** process tabulate output for aggregation type 'R' ****;
	   if substr(report_variable_format,1,1) NE '$' and upcase(percentile_report)='Y' and aggregation_type in('R') then do;
	       call execute("proc datasets library=repout nolist nodetails;");
	       call execute("modify "||strip(outputds_prefix)||"_tab_var_"||strip(varcount)||";");
	       call execute("rename ");
		     do j=1 to dim(one); ** generates the time period vars **;
	            call execute(strip(one{j})||"=_D"||strip(j)||"_");
	         end;
		   call execute(";");
		   call execute("label "||strip(report_variable)||"=' ';");
		   call execute("run;quit;");
	   end;
   
END; *** END if type in('DEV_DAILY','DEV_MONTHLY')***;

*--------------------------------------------------------------------------*
| Process Output Proc Report Data Sets for 'Daily' 'Monthly'               |
*--------------------------------------------------------------------------*;

if type IN('DAILY','MONTHLY') AND last.report_variable And Tablecount>=1  THEN DO;
	call execute("Data work."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";");
	call execute("Set work."||strip(outputds_prefix)||"_report_var_"||strip(varcount)||";");
	call execute("Drop _BREAK_;");
	call execute("If _BREAK_='_RBREAK_' then delete;");
	IF aggregation_type in('G','R') then do;
	    call execute("label _C2_='"||strip(aggregation_result_label)||"';");
	    call execute("label _C4_='"||strip(aggregation_result_label)||"';");
	    if type='DAILY' then call execute("label _C6_='"||strip(aggregation_result_label)||"';");
	    call execute("rename group="||strip(report_variable)||";");
            call execute("format group " || strip(report_variable_format) || ";");
    	end;
	else do;
	    call execute("label _C2_='Count';");
	    call execute("label _C4_='Count';");
	    if type='DAILY' then call execute("label _C6_='Count';");
            call execute("format " || strip(report_variable) || " " || strip(report_variable_format) || ";");
	end;
	   call execute("label _C3_='Percent';");
	   call execute("label _C5_='Percent';");
	   if type='DAILY' then call execute("label _C7_='Percent';");
	call execute("run;");

	 **** process tabulate output  for aggregation type 'R' ****;
     if substr(report_variable_format,1,1) NE '$' and upcase(percentile_report)='Y' and aggregation_type in('R')  then do;
	     call execute("proc datasets library=work nolist nodetails;");
	     call execute("modify "||strip(outputds_prefix)||"_tab_var_"||strip(varcount)||";");
	     call execute("rename ");
		   do j=1 to dim(one); ** generates the time period vars **;  k=dim(one)+1-j;
	          call execute(strip(one{j})||"=_C"||strip(k)||"_");
	       end;
		 call execute(";");
		  call execute("label "||strip(report_variable)||"=' ';");
	     call execute("run;quit;");
	 end;

END;
**** clear the titles and any potential footnote ***;

if last.report_variable then do;  
   if title_number>=1 then do j=1 to title_number;
     call execute("title"||strip(j)||";");
   end;
END;

run;  

ods pdf close;
 
%mend main_report_outputds;
