/*
 * extract development for analytic reporting
 * 
 * Author: Song, Qiang <qiang.song@sas.com> 2015
 * 
 */

/* software version 4.1: refer to documentation Report History Database Dictionary */
%let smh_activity_type_BRST = ST;

%let smh_activity_type_CCBF = "BF";
%let smh_activity_type_CCCA = "CA";
%let smh_activity_type_CCCP = "CP";
%let smh_activity_type_CCDP = "DP";
%let smh_activity_type_CCMF = "M2", "M3", "MC", "MR";
%let smh_activity_type_CCNM = "UK", "NM";
%let smh_activity_type_CCSH = "SH";
%let smh_activity_type_CCAB = "AB";
%let smh_activity_type_CCAE = "AE";

%let smh_activity_type_CSBF = "BF";
%let smh_activity_type_CSCA = "CA";
%let smh_activity_type_CSCK = "CK";
%let smh_activity_type_CSCP = "CP";
%let smh_activity_type_CSCW = "CW";
%let smh_activity_type_CSDP = "DP";
%let smh_activity_type_CSMF = "M3", "MC", "MD";
%let smh_activity_type_CSNM = "UK", "NM";
%let smh_activity_type_CSSH = "SH";

%let smh_activity_type_LCBF = "BF";
%let smh_activity_type_LCCA = "CA";
%let smh_activity_type_LCCK = "CK";
%let smh_activity_type_LCCP = "CP";
%let smh_activity_type_LCCW = "CW";
%let smh_activity_type_LCMF = "MF", "ML";
%let smh_activity_type_LCNM = "UK", "NM";

%let smh_activity_type_NAME = "ME";
%let smh_activity_type_NAMF = "M3", "MB", "MC", "MI";
%let smh_activity_type_NAMO = "MO";
%let smh_activity_type_NAMP = "MP";
%let smh_activity_type_NANM = "UK", "NM";

%let smh_activity_type_SLCK = "CK";
%let smh_activity_type_SLCP = "CP";
%let smh_activity_type_SLBF = "BF";
%let smh_activity_type_SLCA = "CA";
%let smh_activity_type_SLCW = "CW";
%let smh_activity_type_SLDP = "DP";
%let smh_activity_type_SLMF = "MC", "MS";
%let smh_activity_type_SLNM = "UK", "NM";
%let smh_activity_type_SLSH = "SH";


%macro rpt_extract_development_data(
    source_dst,           /* dataset of development data */ 
    frh_table,            /* CCCA, CSCA, etc */
    type,                 /* daily, monthly */
    wherecond = 1,        /* conditions to subsetting input dataset */    
    wts_var = 1,          /* weight of each observation */    
    out_dst =             /* the output dataset will be default to work.&frh_table; */ 
    );

    %let out_dst = %sysfunc(ifc("&out_dst" = "", &frh_table, &out_dst));

    %local acct_type activity_type;
    %let acct_type = %sysfunc(substr(&frh_table, 1, 2));
    %let activity_type = %sysfunc(substr(&frh_table, 3, 2));

    libname devdata "&report_root/development_data/" access = readonly;

    %local acct_type_exist activity_type_exist;    
    data _null_;
        dsid = open("&source_dst");

        f = ifc(varnum(dsid, "smh_acct_type"),"Y", "N");
        call symputx('acct_type_exist', f);

        f = ifc(varnum(dsid, "smh_activity_type"),"Y", "N");
        call symputx('activity_type_exist', f);
    run;

    %local var_list;
    proc sql;
        select var_list into :var_list
            from devdata.frh_table_info_&type.
            where frh_table = "&frh_table";
    quit;
    
    data &out_dst;
        set &source_dst(
            keep = &var_list
                   %if &acct_type_exist = Y %then %do; smh_acct_type %end; 
                   %if &activity_type_exist = Y %then %do; smh_activity_type %end;
            );
                       
        where &wherecond
                       
            %if &acct_type_exist = Y %then %do;
                and smh_acct_type = "&acct_type"  
            %end; 
            
            %if &activity_type_exist = Y %then %do;
                and smh_activity_type in (&&smh_activity_type_&frh_table)
            %end; 
        ; /* ENDING WHERE */
            
        length wts_var 8;
        wts_var = &wts_var;  
    run;

    libname devdata clear;
%mend;

     
